package com.ichaoj.ams.controller;

import cn.hutool.core.bean.BeanUtil;
import com.ichaoj.ams.entity.AmsAirdropTask;
import com.ichaoj.ams.request.airdrop.PageProjectRequest;
import com.ichaoj.ams.request.task.CreateAirdropTask;
import com.ichaoj.ams.request.task.PageTaskRequest;
import com.ichaoj.ams.request.task.UpdateAirdropTask;
import com.ichaoj.ams.request.task.UpdateTaskStatusRequest;
import com.ichaoj.ams.response.airdrop.AirdropProjectResponse;
import com.ichaoj.ams.response.task.TaskResponse;
import com.ichaoj.ams.service.IAmsAirdropTaskService;
import com.ichaoj.common.annotation.AuthResource;
import com.ichaoj.common.exception.ErrorServiceException;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.context.annotation.Lazy;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * <p>
 * 空投项目任务 前端控制器
 * </p>
 *
 * @author cjwen
 * @since 2023-04-25
 */
@RestController
@RequestMapping("/airdrop-task")
@Tag(name = "任务模块")
public class AirdropTaskController {

    @Resource
    private IAmsAirdropTaskService taskService;

    @PostMapping
    @Operation(summary = "创建空投任务")
    @AuthResource(true)
    public PublicResult<Object> createAirdropTask(@RequestBody CreateAirdropTask createAirdropTask) {
        taskService.save(BeanUtil.copyProperties(createAirdropTask, AmsAirdropTask.class));
        return PublicResult.success();
    }

    @PutMapping
    @Operation(summary = "修改空投任务")
    @AuthResource(true)
    public PublicResult<Object> updateAirdropTask(@RequestBody @Validated UpdateAirdropTask updateAirdropTask) {
        taskService.updateById(BeanUtil.copyProperties(updateAirdropTask, AmsAirdropTask.class));
        return PublicResult.success();
    }

    @PutMapping("update-status")
    @Operation(summary = "修改空投任务状态")
    @AuthResource
    public PublicResult<Object> updateAirdropTaskStatus(@RequestBody @Validated UpdateTaskStatusRequest statusRequest) {
        AmsAirdropTask task = taskService.getById(statusRequest.getAmsTaskId());
        if (task != null) {
            task.setTaskStatus(statusRequest.getTaskStatus());
            taskService.updateById(task);
        }else {
            throw new ErrorServiceException("Task Error");
        }
        return PublicResult.success();
    }

    @DeleteMapping("/{taskId}")
    @Operation(summary = "删除空投任务")
    @AuthResource(true)
    public PublicResult<Object> deleteAirdropTask(@PathVariable String taskId) {
        taskService.removeById(taskId);
        return PublicResult.success();
    }

    @PostMapping("/page")
    @Operation(summary = "分页查询空投任务")
    @AuthResource
    public PublicResult<PublicPage<TaskResponse>> pageTask(@RequestBody PageTaskRequest taskRequest) {
        PublicPage<TaskResponse> result = taskService.pageAirdropProject(taskRequest);
        return PublicResult.success(result);
    }
}
